// chrome://extensions
// check “Developer mode” to enable loading unpacked extensions
// This will allow you to load your extension from a folder. Finally, click “Load unpacked extension” or simply drag the “Timesheet” folder onto the page to load up the extension


function debounce(timeout, callback) {
  let timeoutID = 0;
  return (event) => {
    clearTimeout(timeoutID);
    timeoutID = setTimeout(() => callback(event), timeout);
  };
}

// ------------------
// .start / .stop
// ------------------

const colorIcon = (parent) => {
	chrome.storage.sync.get(['timer_stopped'], function(items) {
	  const actionStopped = items.timer_stopped;
	  let manifest = chrome.runtime.getManifest();
	  const select = document.getElementById('popup-options');
    const select2 = document.getElementById('popup-tasks');
    const notes = document.getElementById('notes');
	  if (actionStopped) { // Start
		  chrome.action.setIcon({ path: manifest.action.start_icon });
		  select.disabled = true;
      select2.disabled = true;
      notes.disabled = true;
	  } else {
		  chrome.action.setIcon({ path: manifest.action.stop_icon });
		  select.disabled = false;
      select2.disabled = false;
      notes.disabled = false;
	  }
  });
}


document.getElementById('toggle-state-button').addEventListener('click', (_event) => {
  chrome.storage.sync.get(['timer_stopped'], async function(items) {
    const actionStopped = items.timer_stopped;
    if (actionStopped) { // is gestopt
      await timerStop();
      await saveLocal({timer_stopped: false});
      colorIcon();
    } else {
      await timerStart();
      await saveLocal({timer_stopped: true});
      colorIcon();
    }
  });
});
const saveLocal = async (s) => {
  chrome.storage.sync.set(s);
  return true;
}
const getLocal = async (key) => {
  return new Promise((resolve, reject) => {
    chrome.storage.sync.get([key], function (result) {
      resolve(result[key]==null?'':result[key]);
    });
  });
}

// ------------------
// .selects
// ------------------

let currentProjectId = '';
document.getElementById('popup-options').addEventListener('change', async (event) => {
  currentProjectId = event.target.value;
  saveLocal({timer_project_id: currentProjectId});
});

let currentTaskId = '';
document.getElementById('popup-tasks').addEventListener('change', async (event) => {
  currentTaskId = event.target.value;
  saveLocal({timer_task_id: currentTaskId});
});

// ------------------
// .notes
// ------------------

let currentNote = '';
document.getElementById('notes').addEventListener('change', async (event) => {
  currentNote = event.target.value;
});

// ------------------
// .facture resources
// ------------------

let baseAPI = "";
let username = "";
let password = "";
let token = '';
let projects = [];
let tasks = [];

const getToken = async (endPoint) => {
  let path = `${baseAPI}api/token/`;
  const fetchResult = await fetch(path, {
    method: 'post',
    body: JSON.stringify({
      'email':username,
      'password':password
    })
  });
  const result = await fetchResult.json();

  if (fetchResult.ok) {
    token = result.token;
    return result.token;
  }
  const responseError = {
    type: "Error",
    message: result.message || "Something went wrong",
    data: result.data || "",
    code: result.code || "",
  };

  let error = new Error();
  error = { ...error, ...responseError };
  throw error;
};

const getProjects = async (endPoint) => {
  const selected = await getLocal('timer_project_id');
  token = await getToken();
  let path = `${baseAPI}api/timesheet/projects/`;
  const fetchResult = await fetch(path, {
    method: 'GET',
    headers: {
      'Authorization': 'Bearer ' + token ,
      'Content-Type': 'application/json'
    }
  });
  const result = await fetchResult.json();
  let first = true;

  if (fetchResult.ok) {
    projects = result.data;
    removeOptions('popup-options');
    Object.entries(projects).forEach(entry => {
      const [key, value] = entry;
      appendOption('popup-options', value.titel, value.project_id,value.project_id==selected?1:0);
      if (first) {
        currentProjectId = value.project_id;
        first = false;
      }
    });
    return result;
  }
  const responseError = {
    type: "Error",
    message: result.message || "Something went wrong",
    data: result.data || "",
    code: result.code || "",
  };

  let error = new Error();
  error = { ...error, ...responseError };
  throw error;
};

const getTasks = async (endPoint) => {
  const selected = await getLocal('timer_task_id');

  token = await getToken();
  let path = `${baseAPI}api/timesheet/tasks/`;
  const fetchResult = await fetch(path, {
    method: 'GET',
    headers: {
      'Authorization': 'Bearer ' + token ,
      'Content-Type': 'application/json'
    }
  });
  const result = await fetchResult.json();
  let first = true;

  if (fetchResult.ok) {
    projects = result.data;
    // reset all options
    removeOptions('popup-tasks');
    Object.entries(projects).forEach(entry => {
      const [key, value] = entry;
      appendOption('popup-tasks', value.titel, value.task_id, value.task_id==selected?1:0);
      if (first) {
        currentTaskId = value.task_id;
        first = false;
      }
    });

    return result;
  }
  const responseError = {
    type: "Error",
    message: result.message || "Something went wrong",
    data: result.data || "",
    code: result.code || "",
  };

  let error = new Error();
  error = { ...error, ...responseError };
  throw error;
};

const timerStart = async (endPoint) => {
  token = await getToken();
  const now = new Date();
  const created = now.getFullYear()+'-'+(now.getMonth()+1)+'-'+now.getDate() + ' ' + now.getHours() + ":" + now.getMinutes() + ":" + now.getSeconds();

  let path = `${baseAPI}api/timesheet/start/`;
  const json = {
    "project_id": currentProjectId,
    "task_id": currentTaskId,
    "created": created,
    "note": currentNote,
  };
  const fetchResult = await fetch(path, {
    method: 'POST',
    headers: {
      'Authorization': 'Bearer ' + token ,
      'Content-Type': 'application/json'
    },
    body: JSON.stringify(json)
  });
  const result = await fetchResult.json();

  if (fetchResult.ok) {
    return result;
  }
  const responseError = {
    type: "Error",
    message: result.message || "Something went wrong",
    data: result.data || "",
    code: result.code || "",
  };

  let error = new Error();
  error = { ...error, ...responseError };
  throw error;
};

const timerStop = async (endPoint) => {
  token = await getToken();

  let path = `${baseAPI}api/timesheet/stop/`;
  const fetchResult = await fetch(path, {
    method: 'POST',
    headers: {
      'Authorization': 'Bearer ' + token ,
      'Content-Type': 'application/json'
    }
  });
  const result = await fetchResult.json();

  if (fetchResult.ok) {
    return result;
  }
  const responseError = {
    type: "Error",
    message: result.message || "Something went wrong",
    data: result.data || "",
    code: result.code || "",
  };

  let error = new Error();
  error = { ...error, ...responseError };
  throw error;
};

// ------------------
// .utils
// ------------------
const removeOptions = (parent) => {
  var elSel = parent;
  if (typeof(parent)!='object') elSel = document.getElementById(parent);
  if (elSel.length > 0){
    for (var i=elSel.length-1; i>=0;i--) elSel.remove(i);
  }
};
const appendOption = (parent,text,value,selected) => {
  var elSel = parent;
  if (typeof(parent)!='object') elSel = document.getElementById(parent);
  var elOptNew = document.createElement('option');
  elOptNew.text = text;
  elOptNew.value = value;
  if (selected == 1) {
    elOptNew.selected = true;
  }
  try {
    elSel.add(elOptNew, null); // standards compliant; doesn't work in IE
  }
  catch(ex) {
    elSel.add(elOptNew); // IE only
  }

};

const start = async () => {
  baseAPI = await getLocal('defacture_apiUrl');
  username = await getLocal('defacture_username');
  password = await getLocal('defacture_password');
  if (baseAPI.length==0) alert('Please fill in the settings - right click the extention > options');
  else {
    colorIcon();
    projects = await getProjects();
    tasks = await getTasks();
  }

}
start();